-- To use with switches attached to pullup resistors
-- 0 is only ever caused by deliberate user action
-- 1 is caused by inaction or temporary "bounce"
-- Therefore 0 state is taken seriously, but 1 must be earned.


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Debounce is
	generic ( 
		MAX_COUNT : integer := 96 * 500;  --500us @ 96Mhz
		HIGH_STEP : integer := 1; -- Speed to head towards '1'
		LOW_STEP : integer := 1 -- Speed to head towards '0'
		);
    Port ( clk : in  STD_LOGIC;
           sigin : in  STD_LOGIC;
           sigout : out  STD_LOGIC);
end Debounce;

architecture Behavioral of Debounce is
	signal sigout_int : STD_LOGIC;

	signal count  : integer range 0 to MAX_COUNT := MAX_COUNT/2;
	signal sigin_delayed : std_logic_vector(2 downto 0) := (others => '0');  --3 stage sampling (for stability)
begin
	sigout <= sigout_int;

	CaptureInputProcess : process (clk)
	begin
		if  rising_edge(clk)  then --Clock
			-- Sample
			sigin_delayed <= sigin_delayed(1 downto 0) & sigin;
			-- Count & output
			if (sigin_delayed(2)='1') then
				if (HIGH_STEP = 1) then
					if (count = MAX_COUNT) then
						sigout_int <= '1';
					else
						count <= count+1;
					end if;
				else
					if (count > MAX_COUNT-HIGH_STEP) then
						count <= MAX_COUNT;
						sigout_int <= '1';
					else
						count <= count+HIGH_STEP;
					end if;
				end if;
			else
				if (LOW_STEP = 1) then
					if (count = 0) then
						sigout_int <= '0';
					else
						count <= count-LOW_STEP;
					end if;
				else
					if (count < LOW_STEP) then
						count <= 0;
						sigout_int <= '0';
					else
						count <= count-LOW_STEP;
					end if;
				end if;
			end if;
		end if;
	end process;

end Behavioral;